/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.geoviewer.jxmap.map.cache.spi;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Pattern;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;

/**
 * CacheManager holds records for all cached (serialised) tiles.
 *
 * <p> During {@link #refresh()} scans cache directory and save tiles records by
 * zoom and coordinates. </p>
 *
 * @author Frantisek Post
 */
public class CacheManager {

    
    private static final String CACHE_ROOT_DIR = "tileCache";
    private static final Pattern DIR_PATTERN = Pattern.compile("\\d*");
    private static final Pattern CACHE_FILE_PATTERN = Pattern.compile("\\d*x\\d*");
    private static final CacheManager instance = new CacheManager();
    private static final Logger LOGGER = Logger.getLogger(CacheManager.class.getName()); 
    
    //for faster searching - key is zoom level, value is map. In that map, key is X coord. and value is list of Y coords.
    //TODO check memory cost
    private Map<Integer, Map<Integer, List<Integer>>> cachedTilesMap;
    private long cacheSizeSum = 0;

    protected CacheManager() {
        cachedTilesMap = new HashMap<>();
        refresh();
    }

    public static CacheManager getInstance() {
        return instance;
    }

    public void refresh() {
        cachedTilesMap.clear();
        cacheSizeSum = 0;

        FileObject cacheFolder = getCacheFolder();
        if (cacheFolder != null) {
            FileObject[] zoomFolders = cacheFolder.getChildren();
            for (FileObject child : zoomFolders) {
                if (child.isValid() && child.isFolder() && DIR_PATTERN.matcher(child.getName()).matches()) { 
                    proccessFolder(child);
                }
            }
        }
    }

    public boolean isCached(int zoom, int x, int y) {
        Map<Integer, List<Integer>> coordMap = cachedTilesMap.get(zoom);
        if (coordMap != null) {
            List<Integer> yCoordList = coordMap.get(x);
            if (yCoordList != null) {
                return yCoordList.contains(y);
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    protected void proccessFolder(FileObject folder) {
        int zoom = Integer.parseInt(folder.getName());
        FileObject[] children = folder.getChildren();
        for (FileObject child : children) {
            if (child.isData() && CACHE_FILE_PATTERN.matcher(child.getName()).matches()) { 
                proccessFile(zoom, child);
            }
        }
    }

    protected void proccessFile(int zoom, FileObject file) {
        String name = file.getName();
        String[] coords = name.split("x");
        if (coords != null && coords.length == 2) {
            int x = Integer.parseInt(coords[0]);
            int y = Integer.parseInt(coords[1]);
            addRecord(zoom, x, y);
        }
    }

    protected void addFileSize(FileObject file) {
        cacheSizeSum += file.getSize();
    }

    protected void addRecord(int zoom, int x, int y) {
        Map<Integer, List<Integer>> zoomMap = cachedTilesMap.get(zoom);
        if (zoomMap == null) {
            zoomMap = new HashMap<>();
            cachedTilesMap.put(zoom, zoomMap);
        }
        List<Integer> yCoords = zoomMap.get(x);
        if (yCoords == null) {
            yCoords = new ArrayList<>();
            zoomMap.put(x, yCoords);
        }
        yCoords.add(y);
    }

    public long getCacheSize() {
        return cacheSizeSum;
    }
    
    public FileObject getCacheFolder() {
        FileObject configRoot = FileUtil.getConfigRoot();
        FileObject configFolder = configRoot.getFileObject(CACHE_ROOT_DIR);
        if (null == configFolder) {
            try {
                configFolder = configRoot.createFolder(CACHE_ROOT_DIR);
            } catch (IOException ex) {
                LOGGER.log(Level.FINE, "Exception while getting tile chache folder.", ex);
            }
        }
        return configFolder;
    }
    
}
